package com.gic.finance.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.dto.InvoiceAccountDTO;
import com.gic.finance.entity.TabInvoiceAccount;
import com.gic.finance.service.InvoiceAccountApiService;
import com.gic.finance.service.InvoiceAccountService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

@Service("invoiceAccountApiService")
public class InvoiceAccountApiServiceImpl implements InvoiceAccountApiService {
    @Autowired
    private InvoiceAccountService invoiceAccountService;

    @Override
    public ServiceResponse<Void> save(InvoiceAccountDTO invoiceAccountDTO) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(invoiceAccountDTO, InvoiceAccountDTO.Save.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        //关闭之前的
        invoiceAccountService.closeStatus();
        invoiceAccountDTO.setCreateTime(new Date());
        invoiceAccountDTO.setUpdateTime(new Date());
        //启用默认
        invoiceAccountDTO.setStatus(1);
        invoiceAccountService.save(invoiceAccountDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> update(InvoiceAccountDTO invoiceAccountDTO) {
        ServiceResponse paramValid = ValidParamsUtils.allCheckValidate(invoiceAccountDTO, InvoiceAccountDTO.Save.class,
                InvoiceAccountDTO.Edit.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        TabInvoiceAccount tab = invoiceAccountService.getById(invoiceAccountDTO.getInvoiceAccountId());
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        invoiceAccountService.update(invoiceAccountDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> enable(Integer id) {
        TabInvoiceAccount tab = invoiceAccountService.getById(id);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        //关闭之前的
        invoiceAccountService.closeStatus();
        InvoiceAccountDTO dto = new InvoiceAccountDTO();
        dto.setInvoiceAccountId(id);
        dto.setStatus(1);
        invoiceAccountService.update(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<InvoiceAccountDTO> getById(Integer invoiceAccountId) {
        TabInvoiceAccount tab = invoiceAccountService.getById(invoiceAccountId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceAccountDTO.class, tab));
    }

    @Override
    public ServiceResponse<InvoiceAccountDTO> getEnable() {
        TabInvoiceAccount tab = invoiceAccountService.getEnable();
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有开启数据，请添加");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceAccountDTO.class, tab));
    }
}
