package com.gic.finance.service.outer.impl;

import java.util.Date;

import com.gic.finance.entity.TabInvoiceAccount;
import com.gic.finance.service.InvoiceAccountService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.constant.InvoiceStatusEnum;
import com.gic.finance.dto.InvoiceManageDTO;
import com.gic.finance.entity.TabInvoiceManage;
import com.gic.finance.qo.InvoiceManageListQueryQO;
import com.gic.finance.service.InvoiceManageApiService;
import com.gic.finance.service.InvoiceManageService;
import com.gic.store.utils.CreateSerialNumberUtils;

@Service("invoiceManageApiService")
public class InvoiceManageApiServiceImpl implements InvoiceManageApiService{
    @Autowired
    private InvoiceManageService invoiceManageService;
    @Autowired
    private InvoiceAccountService invoiceAccountService;
    @Override
    public ServiceResponse<Void> applyInvoice(InvoiceManageDTO invoiceManageDTO) {
        ServiceResponse paramResponse = ValidParamsUtils.allCheckValidate(invoiceManageDTO, InvoiceManageDTO.ApplyInvoiceValid.class);
        if (!paramResponse.isSuccess()) {
            return paramResponse;
        }
        invoiceManageDTO.setCreateTime(new Date());
        invoiceManageDTO.setUpdateTime(new Date());
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.TO_BE_ISSUED.getCode());
        invoiceManageDTO.setInvoiceApplySerial(CreateSerialNumberUtils.createSerialNumber());

        invoiceManageService.save(invoiceManageDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> invoice(Integer invoiceManageId, String expressMailName, String expressMailNumber) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        if (StringUtils.isBlank(expressMailName) || StringUtils.isBlank(expressMailNumber)) {
            //待邮寄
            invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.TO_BE_MAILED.getCode());
        } else {
            invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.MAILED.getCode());
            //物流信息
            invoiceManageDTO.setExpressMailName(expressMailName);
            invoiceManageDTO.setExpressMailNumber(expressMailNumber);
        }
        //todo 操作人相关信息

        //开票户信息插入
        TabInvoiceAccount account = invoiceAccountService.getEnable();
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "开票户信息未设置，请前往添加");
        }
        invoiceManageDTO.setInvoicerId(account.getInvoiceAccountId());
        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rejectInvoice(Integer invoiceManageId, String rejectReason) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.REJECT.getCode());
        if (StringUtils.isBlank(rejectReason)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "驳回理由不能为空");
        }
        invoiceManageDTO.setRejectReason(rejectReason);
        //todo 操作人相关信息

        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelInvoice(Integer invoiceManageId) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.CANCEL.getCode());
        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<InvoiceManageDTO>> listInvoiceManage(InvoiceManageListQueryQO params) {
        com.github.pagehelper.Page page = invoiceManageService.listInvoiceManage(params);
        Page<InvoiceManageDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, InvoiceManageDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<InvoiceManageDTO> getById(Integer invoiceManageId) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceManageDTO.class, tabInvoiceManage));
    }
}
