package com.gic.finance.service.outer.impl;

import java.text.SimpleDateFormat;
import java.util.Date;

import com.alibaba.fastjson.JSONObject;
import com.gic.dubbo.util.DubboContextUtil;
import com.gic.dubbo.util.DubboInvokeUtil;
import org.apache.dubbo.config.ApplicationConfig;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.config.RegistryConfig;
import org.apache.dubbo.config.utils.ReferenceConfigCache;
import org.apache.dubbo.rpc.service.GenericService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.finance.constant.OrderTypeEnum;
import com.gic.finance.constant.TransferAccountApprovalStatusEnum;
import com.gic.finance.dto.TransferAccountsApprovalDTO;
import com.gic.finance.entity.TabTransferAccountsApproval;
import com.gic.finance.qo.TransferListQueryQO;
import com.gic.finance.service.TransferAccountsApprovalApiService;
import com.gic.finance.service.TransferAccountsApprovalService;
import com.gic.store.utils.CreateSerialNumberUtils;
import com.gic.store.utils.valid.ValidUtil;

/**
 * 转账审批
 * @author $user$
 * @ClassName: $ClassName$
 * @Description: $desc$
 * @date $date$ $time$
 */
@Service("transferAccountsApprovalApiService")
public class TransferAccountsApprovalApiServiceImpl implements TransferAccountsApprovalApiService {
    private static final Logger LOGGER = LoggerFactory.getLogger(TransferAccountsApprovalApiServiceImpl.class);
    private final static String SERVICE_NAME = "转账审批";
    @Autowired
    private TransferAccountsApprovalService transferAccountsApprovalService;

    /**
     * 发起审批
     * @Title: initiateApproval 
     * @Description:
      * @author guojuxing
     * @param transferAccountsApprovalDTO 
     * @return  com.gic.api.base.commons.ServiceResponse<java.lang.Void> 
      */
    private ServiceResponse<Void> initiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        //valid param
        ServiceResponse paramValid = ValidUtil.allCheckValidate(transferAccountsApprovalDTO,
                TransferAccountsApprovalDTO.InitiateApprovalValid.class);
        if (!paramValid.isSuccess()) {
            return paramValid;
        }
        transferAccountsApprovalDTO.setCreateTime(new Date());
        transferAccountsApprovalDTO.setUpdateTime(new Date());
        transferAccountsApprovalDTO.setApprovalStatus(TransferAccountApprovalStatusEnum.TO_BE_APPROVED.getCode());
        //审批流水号生成
        String approvalSerialNumber = CreateSerialNumberUtils.createSerialNumber();
        LOGGER.info("转账审批-发起审批-审批流水号：{}", approvalSerialNumber);
        transferAccountsApprovalDTO.setApprovalNumber(approvalSerialNumber);

        transferAccountsApprovalService.initiateApproval(transferAccountsApprovalDTO);

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rechargeInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.RECHARGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> smsInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.SHORT_MESSAGE_PACKAGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> serviceInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.SERVICE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> extensionPackageInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.EXTENSION_PACKAGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<Void> packageInitiateApproval(TransferAccountsApprovalDTO transferAccountsApprovalDTO) {
        transferAccountsApprovalDTO.setOrderType(OrderTypeEnum.PACKAGE.getCode());
        return initiateApproval(transferAccountsApprovalDTO);
    }

    @Override
    public ServiceResponse<TransferAccountsApprovalDTO> getById(Integer transferApprovalId) {
        LOGGER.info(SERVICE_NAME + "-getById:{}", transferApprovalId);
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(TransferAccountsApprovalDTO.class, tab));
    }

    @Override
    public ServiceResponse<TransferAccountsApprovalDTO> getByOrderNumber(String orderNumber) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getByOrderNumber(orderNumber);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "订单流水号有误，查询不到数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(TransferAccountsApprovalDTO.class, tab));
    }

    @Override
    public ServiceResponse<Void> agreeApproval(Integer transferApprovalId, Double accountAmount) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        if (accountAmount == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "到账金额不能为空");
        }
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setAccountAmount(accountAmount);
        dto.setBadAmount(tab.getPlannedAmount() - accountAmount);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.AGREE.getCode());
        dto.setApprovalTime(new Date());
        //todo 审批人等信息获取插入

        //callBack(tab, accountAmount, TransferAccountApprovalStatusEnum.AGREE.getCode());
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rejectApproval(Integer transferApprovalId) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.REJECT.getCode());
        dto.setApprovalTime(new Date());
        //todo 审批人等信息获取插入

        //callBack(tab, null, TransferAccountApprovalStatusEnum.AGREE.getCode());
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelApproval(String orderSerialNumber) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getByOrderNumber(orderSerialNumber);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "订单流水号有误");
        }
        updateCancelApproval(tab.getTransferApprovalId());
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelApproval(Integer transferApprovalId) {
        TabTransferAccountsApproval tab = transferAccountsApprovalService.getById(transferApprovalId);
        if (tab == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "转账审批主键有误");
        }
        updateCancelApproval(transferApprovalId);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<TransferAccountsApprovalDTO>> listTransferAccountsApproval(TransferListQueryQO params) {
        com.github.pagehelper.Page page = transferAccountsApprovalService.listTransferAccountsApproval(params);
        Page<TransferAccountsApprovalDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, TransferAccountsApprovalDTO.class);
        return ServiceResponse.success(resultPage);
    }

    /**
     * 状态改为取消
     * @Title: updateCancelApproval 
     * @Description:
      * @author guojuxing
     * @param transferApprovalId 
     * @return  void 
      */
    private void updateCancelApproval(Integer transferApprovalId) {
        TransferAccountsApprovalDTO dto = new TransferAccountsApprovalDTO();
        dto.setTransferApprovalId(transferApprovalId);
        dto.setApprovalStatus(TransferAccountApprovalStatusEnum.CANCEL.getCode());
        transferAccountsApprovalService.updateTransferAccountsApproval(dto);
    }

    private void callBack(TabTransferAccountsApproval tab, Double accountAmount, int status) {
        LOGGER.info("回调了");
        ApplicationConfig applicationConfig = new ApplicationConfig();
        applicationConfig.setName("dubbo-job");

        RegistryConfig registryConfig = new RegistryConfig();
        registryConfig.setAddress("zookeeper://10.105.220.75:2199");
        registryConfig.setProtocol("dubbo");
        registryConfig.setId("customRegistry");
        registryConfig.setTimeout(30000);


        ReferenceConfig<GenericService> reference = new ReferenceConfig<GenericService>();
        reference.setApplication(applicationConfig);
        reference.setRegistry(registryConfig);
        reference.setInterface("com.gic.enterprise.service.BillingPayInfoApiService");
        reference.setGeneric(true);

        reference.setCheck(false);
        reference.setAsync(false);
        reference.setTimeout(50000);
        reference.setRetries(0);
        reference.setGeneric(true);
        ReferenceConfigCache cache = ReferenceConfigCache.getCache();
        GenericService genericService = cache.get(reference);


        boolean isBilling = tab.getOrderType().intValue() == OrderTypeEnum.RECHARGE.getCode() ||
                tab.getOrderType().intValue() == OrderTypeEnum.SHORT_MESSAGE_PACKAGE.getCode();
        //回调计费中心、短信套餐包
        if (isBilling) {
            JSONObject param = new JSONObject();
            param.put("orderSerialNumber", tab.getOrderNumber());
            param.put("buyType", tab.getOrderType());
            param.put("totalFeePaid", accountAmount);
            param.put("auditStatus", status);
            param.put("timeEnd", new SimpleDateFormat("yyyyMMdd").format(new Date()));
            //DubboContextUtil.initRegistryConfig("10.105.220.75", 2199);
            Object name = genericService.$invoke("offlineCallBack", new String[]{String.class.getName()}, new String[]{param.toJSONString()});
        }
    }
}
