package com.gic.finance.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.finance.dto.InvoiceAccountDTO;
import com.gic.finance.service.InvoiceAccountApiService;
import com.gic.finance.web.vo.InvoiceManagerDetailVO;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.finance.dto.InvoiceManageDTO;
import com.gic.finance.qo.InvoiceManageListQueryQO;
import com.gic.finance.service.InvoiceManageApiService;
import com.gic.finance.web.vo.InvoiceManageListVO;

/**
 * 发票管理
 * @ClassName: InvoiceManageController 
 * @Description:  
 * @author guojuxing 
 * @date 2019/8/14 3:14 PM 
 */
@RestController
@RequestMapping("/invoice-manage")
public class InvoiceManageController {
    private static Logger LOGGER = LoggerFactory.getLogger(InvoiceManageController.class);
    @Autowired
    private InvoiceManageApiService invoiceManageApiService;
    @Autowired
    private InvoiceAccountApiService invoiceAccountApiService;

    /**
     * 发起审批
     * @param dto
     * @return
     */
    @RequestMapping("/apply-invoice")
    public RestResponse applyInvoice(InvoiceManageDTO dto) {
        return ResultControllerUtils.commonResult(invoiceManageApiService.applyInvoice(dto));
    }

    /**
     * 同意审批
     * @param dto
     * @return
     */
    @RequestMapping("/invoice")
    public RestResponse invoice(InvoiceManageDTO dto) {
        return ResultControllerUtils.commonResult(invoiceManageApiService.invoice(dto.getInvoiceManageId(),
                dto.getExpressMailName(), dto.getExpressMailNumber()));
    }

    /**
     * 驳回审批
     * @param dto
     * @return
     */
    @RequestMapping("/reject-invoice")
    public RestResponse rejectInvoice(InvoiceManageDTO dto) {
        return ResultControllerUtils
                .commonResult(invoiceManageApiService.rejectInvoice(dto.getInvoiceManageId(), dto.getRejectReason()));
    }

    /**
     * 取消审批
     * @param dto
     * @return
     */
    @RequestMapping("/cancel-invoice")
    public RestResponse cancelInvoice(InvoiceManageDTO dto) {
        return ResultControllerUtils.commonResult(invoiceManageApiService.cancelInvoice(dto.getInvoiceManageId()));
    }

    /**
     * 详情
     * @param invoiceManageId
     * @return
     */
    @RequestMapping("/get-detail")
    public RestResponse getDetail(Integer invoiceManageId) {
        ServiceResponse<InvoiceManageDTO> invoiceResult = invoiceManageApiService.getById(invoiceManageId);
        if (invoiceResult.isSuccess()) {
            InvoiceManageDTO invoice = invoiceResult.getResult();
            InvoiceManagerDetailVO vo = EntityUtil.changeEntityNew(InvoiceManagerDetailVO.class, invoice);
            ServiceResponse<InvoiceAccountDTO> accountResult;
            if (invoice.getInvoicerId() != null) {
                accountResult = invoiceAccountApiService.getById(invoice.getInvoicerId());
            } else {
                accountResult = invoiceAccountApiService.getEnable();
            }
            if (accountResult.isSuccess()) {
                InvoiceAccountDTO account = accountResult.getResult();
                vo.setAccountName(account.getAccountName());
                vo.setAccountPhone(account.getAccountPhone());
                vo.setAddress(account.getAddress());
                vo.setBank(account.getBank());
                vo.setBankAccount(account.getBankAccount());
                vo.setTaxNumberOfInvoice(account.getTaxNumber());
            } else {
                return EnterpriseRestResponse.failure(accountResult);
            }
            return RestResponse.success(vo);
        } else {
            return EnterpriseRestResponse.failure(invoiceResult);
        }
    }

    @RequestMapping("/list-invoice")
    public RestResponse listInvoice(InvoiceManageListQueryQO params) {
        return ResultControllerUtils.commonPageResult(
                invoiceManageApiService.listInvoiceManage(params), InvoiceManageListVO.class);
    }

}
